
from vsg import rule
from vsg import fix
from vsg import check


class rule_016(rule.rule):
    '''
    Instantiation rule 016 checks the generic name is uppercase.
    '''

    def __init__(self):
        rule.rule.__init__(self)
        self.name = 'instantiation'
        self.identifier = '016'
        self.solution = 'Uppercase generic name.'
        self.phase = 6

    def analyze(self, oFile):
        for iLineNumber, oLine in enumerate(oFile.lines):
            if oLine.isInstantiationGenericAssignment and not oLine.isInstantiationGenericKeyword:
                check.is_uppercase(self, oLine.line.split()[0], iLineNumber)

    def _fix_violations(self, oFile):
        for iLineNumber in self.violations:
            lLine = oFile.lines[iLineNumber].line.split('=>')
            fix.upper_case(self, oFile.lines[iLineNumber], lLine[0].rstrip().lstrip())



import os
import unittest

from vsg.rules import case
from vsg import vhdlFile


# Read in test file used for all tests
oFile = vhdlFile.vhdlFile(os.path.join(os.path.dirname(__file__),'..','case','case_test_input.vhd'))
oFileSequential = vhdlFile.vhdlFile(os.path.join(os.path.dirname(__file__),'..','case','case_sequential_test_input.vhd'))

class testRuleCaseMethods(unittest.TestCase):

    def test_rule_001(self):
        oRule = case.rule_001()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'case')
        self.assertEqual(oRule.identifier, '001')
        dExpected = [9,33,45,52,70]
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_002(self):
        oRule = case.rule_002()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'case')
        self.assertEqual(oRule.identifier, '002')
        dExpected = [9,77]
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_003(self):
        oRule = case.rule_003()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'case')
        self.assertEqual(oRule.identifier, '003')
        dExpected = [43,77]
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_004(self):
        oRule = case.rule_004()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'case')
        self.assertEqual(oRule.identifier, '004')
        dExpected = [17,29,52]
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_005(self):
        oRule = case.rule_005()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'case')
        self.assertEqual(oRule.identifier, '005')
        dExpected = [17,23,66]
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_006(self):
        oRule = case.rule_006()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'case')
        self.assertEqual(oRule.identifier, '006')
        dExpected = [33]
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_007(self):
        oRule = case.rule_007()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'case')
        self.assertEqual(oRule.identifier, '007')
        dExpected = [9]
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_008(self):
        oRule = case.rule_008()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'case')
        self.assertEqual(oRule.identifier, '008')
        dExpected = [43]
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_009(self):
        oRule = case.rule_009()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'case')
        self.assertEqual(oRule.identifier, '009')
        dExpected = [70]
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_010(self):
        oRule = case.rule_010()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'case')
        self.assertEqual(oRule.identifier, '010')
        dExpected = [33]
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_011(self):
        oRule = case.rule_011()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'case')
        self.assertEqual(oRule.identifier, '011')
        dExpected = [46,59]
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_012(self):
        oRule = case.rule_012()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'case')
        self.assertEqual(oRule.identifier, '012')
        dExpected = [11,12,16]
        oRule.analyze(oFileSequential)
        self.assertEqual(oRule.violations, dExpected)

    def test_rule_013(self):
        oRule = case.rule_013()
        self.assertTrue(oRule)
        self.assertEqual(oRule.name, 'case')
        self.assertEqual(oRule.identifier, '013')
        dExpected = [31]
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)




from vsg.rules import lowercase_word_rule


class rule_004(lowercase_word_rule):
    '''
    Type rule 004 checks the type name is lowercase.
    '''

    def __init__(self):
        lowercase_word_rule.__init__(self, 'type', '004', 'isTypeKeyword', 1)
        self.solution = 'Change type name to lowercase.'



import os
import unittest

from vsg.rules import sequential
from vsg import vhdlFile

# Read in test file used for all tests
oFile = vhdlFile.vhdlFile(os.path.join(os.path.dirname(__file__),'..','sequential','sequential_test_input.vhd'))

class testFixRuleSequentialMethods(unittest.TestCase):

    def test_fix_rule_001(self):
        oRule = sequential.rule_001()
        dExpected = []
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_fix_rule_002(self):
        oRule = sequential.rule_002()
        dExpected = []
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_fix_rule_003(self):
        oRule = sequential.rule_003()
        dExpected = []
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_fix_rule_004(self):
        oRule = sequential.rule_004()
        dExpected = []
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_fix_rule_005(self):
        oRule = sequential.rule_005()
        dExpected = []
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)

    def test_fix_rule_006(self):
        oRule = sequential.rule_006()
        dExpected = []
        oRule.fix(oFile)
        oRule.analyze(oFile)
        self.assertEqual(oRule.violations, dExpected)




from rule_001 import rule_001
from rule_002 import rule_002
from rule_003 import rule_003
from rule_004 import rule_004
from rule_005 import rule_005
from rule_006 import rule_006
from rule_007 import rule_007
from rule_008 import rule_008
from rule_009 import rule_009
from rule_010 import rule_010
from rule_011 import rule_011
from rule_012 import rule_012
from rule_013 import rule_013
from rule_014 import rule_014
from rule_015 import rule_015

#  ordering rules to handle compressed if statements
from rule_020 import rule_020  # end if
from rule_021 import rule_021  # else
from rule_022 import rule_022  # code after else
from rule_023 import rule_023  # elsif
from rule_024 import rule_024  # code after then

from rule_025 import rule_025
from rule_026 import rule_026
from rule_027 import rule_027
from rule_028 import rule_028
from rule_029 import rule_029
